@@ -2,6 +2,7 @@ from pytest import raises
 from notario import engine
 from notario.exceptions import Invalid, SchemaError
 from notario.validators import recursive, iterables, types
+from notario.decorators import optional
 
 
 class TestEnforce(object):
@@ -10,7 +11,8 @@ class TestEnforce(object):
         def bad_callable(v): assert False
         with raises(Invalid) as exc:
             engine.enforce('1', bad_callable, ['1'], 'key')
-        assert exc.value.args[0] == '-> 1 key did not pass validation against callable: bad_callable'
+        error = exc.value.args[0]
+        assert '-> 1 key did not pass validation against callable: bad_callable' in error
 
     def test_callable_passes_with_flying_colors(self):
         def cool_callable(v): pass
@@ -24,7 +26,8 @@ class TestEnforce(object):
     def test_unequal_params_raise_invalid(self):
         with raises(Invalid) as exc:
             engine.enforce(1, 2, ['1'], 'key')
-        assert exc.value.args[0] == '-> 1 key did not match 2'
+        error = exc.value.args[0]
+        assert '-> 1 key did not match 2' in error
 
     def test_callable_with_messages_are_passed_on(self):
         def callable_message(v): assert False, "this is completely False"
@@ -33,27 +36,6 @@ class TestEnforce(object):
         assert exc.value.reason == "this is completely False"
 
 
-class TestNormalizeSchema(object):
-
-    def test_return_data_structure_with_cero_index(self):
-        result = engine.normalize_schema(['a'])
-        assert result == {0: ['a']}
-
-    def test_recurse_if_second_item_is_typle(self):
-        data = ('a', ('a', 'b'))
-        result = engine.normalize_schema(data)
-        assert result == {0: ('a', {0: ('a', 'b')})}
-
-    def test_respect_more_than_two_values_in_tuple(self):
-        data = ('a', (('a', 'b'), ('c', 'c'), ('d', 'd')))
-        result = engine.normalize_schema(data)
-        assert result == {0: ('a', {0: ('a', 'b'), 1: ('c', 'c'), 2: ('d', 'd')})}
-
-    def test_basic_key_value_pairs(self):
-        data = (('a', 'b'), ('b', 'b'), ('c', 'c'))
-        result = engine.normalize_schema(data)
-        assert result == {0: ('a', 'b'), 1: ('b', 'b'), 2: ('c', 'c')}
-
 class TestValidator(object):
 
     def test_validate_top_level_keys(self):
@@ -63,7 +45,8 @@ class TestValidator(object):
             validator = engine.Validator(data, schema)
             validator.validate()
 
-        assert exc.value.args[0] == "-> b key did not match 'c'"
+        error = exc.value.args[0]
+        assert  "-> b key did not match 'c'" in error
 
     def test_validate_length_equality(self):
         data = {'a': 1, 'b': 2}
@@ -72,16 +55,18 @@ class TestValidator(object):
             validator = engine.Validator(data, schema)
             validator.validate()
 
-        assert exc.value.args[0] == '-> top level length did not match schema'
+        error = exc.value.args[0]
+        assert  '-> top level length did not match schema' in error
 
     def test_validate_length_equality_less_items(self):
         data = {'a': 1, 'b': 2, 'c':'c'}
         schema = (('a', 1), ('b', 2))
-        with raises(SchemaError) as exc:
+        with raises(Invalid) as exc:
             validator = engine.Validator(data, schema)
             validator.validate()
 
-        assert exc.value.args[0] == '-> top level has less items in schema than in data'
+        error = exc.value.args[0]
+        assert "top level has unexpected item in data: ('c', 'c')" in error
 
     def test_validate_length_equality_returns(self):
         data = {0:('a', 1)}
@@ -99,7 +84,8 @@ class TestValidator(object):
             validator = engine.Validator(data, schema)
             validator.validate()
 
-        assert exc.value.args[0] == '-> b -> 2  did not match 1'
+        error = exc.value.args[0]
+        assert  '-> b -> 2  did not match 1' in error
 
     def test_validate_value_nested_dictionaries(self):
         data = {'a': 1, 'b': {'a': 2, 'b' : 1}}
@@ -108,7 +94,8 @@ class TestValidator(object):
             validator = engine.Validator(data, schema)
             validator.validate()
 
-        assert exc.value.args[0] == '-> b -> b -> 1  did not match 2'
+        error = exc.value.args[0]
+        assert  '-> b -> b -> 1  did not match 2' in error
 
     def test_validate_key_doubly_nested_dictionaries(self):
         data = {'a': 1, 'b': {'a': 2, 'b' : {'a':'a'}}}
@@ -117,7 +104,8 @@ class TestValidator(object):
             validator = engine.Validator(data, schema)
             validator.validate()
 
-        assert exc.value.args[0] == "-> b -> b -> a -> a  did not match 'b'"
+        error = exc.value.args[0]
+        assert  "-> b -> b -> a -> a  did not match 'b'" in error
 
     def test_validate_key_nested_dictionaries(self):
         data = {'a': 1, 'b': {'a': 2, 'b' : 1}}
@@ -126,7 +114,8 @@ class TestValidator(object):
             validator = engine.Validator(data, schema)
             validator.validate()
 
-        assert exc.value.args[0] == "-> b -> a key did not match 'b'"
+        error = exc.value.args[0]
+        assert  "-> b -> a key did not match 'b'" in error
 
     def test_validate_value_nested_arrays(self):
         data = {'a': 1, 'b': {'a': 2, 'b' : [1, 2, 3]}}
@@ -135,7 +124,8 @@ class TestValidator(object):
             validator = engine.Validator(data, schema)
             validator.validate()
 
-        assert exc.value.args[0] == '-> b -> b -> [1, 2, 3]  did not match [1, 1, 3]'
+        error = exc.value.args[0]
+        assert  '-> b -> b -> [1, 2, 3]  did not match [1, 1, 3]' in error
 
     def test_validate_key_nested_arrays(self):
         data = {'a': 1, 'b': {'a': 2, 'b' : [1, 2, 3]}}
@@ -144,7 +134,8 @@ class TestValidator(object):
             validator = engine.Validator(data, schema)
             validator.validate()
 
-        assert exc.value.args[0] == '-> b -> b -> [1, 2, 3]  did not match [1, 1, 3]'
+        error = exc.value.args[0]
+        assert  '-> b -> b -> [1, 2, 3]  did not match [1, 1, 3]' in error
 
     def test_validate_multiple_items_as_values(self):
         data = {'a': 1, 'b': {'a': 2, 'b' : 1, 'd':1, 'c':2}}
@@ -153,7 +144,8 @@ class TestValidator(object):
             validator = engine.Validator(data, schema)
             validator.validate()
 
-        assert exc.value.args[0] == '-> b -> d -> 1  did not match 2'
+        error = exc.value.args[0]
+        assert  '-> b -> d -> 1  did not match 2' in error in error
 
 
 class TestValidatorLeaves(object):
@@ -169,7 +161,8 @@ class TestValidatorLeaves(object):
             validator = engine.Validator(data, schema)
             validator.validate()
 
-        assert exc.value.args[0] == '-> a -> b -> list[0] item did not match 2'
+        error = exc.value.args[0]
+        assert  '-> a -> b -> list[0] item did not match 2' in error
 
     def test_traverser_returns_the_recursive_leaf_if_seen(self):
         data = {'a': {'b': [1, 1, 1, 1]}}
@@ -178,7 +171,8 @@ class TestValidatorLeaves(object):
             validator = engine.Validator(data, schema)
             validator.validate()
 
-        assert exc.value.args[0] == '-> a -> b -> [1, 1, 1, 1]  did not match [1, 1, 1, 2]'
+        error = exc.value.args[0]
+        assert  '-> a -> b -> [1, 1, 1, 1]  did not match [1, 1, 1, 2]' in error
 
 
 class TestRecursiveValidator(object):
@@ -190,16 +184,19 @@ class TestRecursiveValidator(object):
         with raises(SchemaError) as exc:
             recursive_validator.validate()
 
-        assert exc.value.args[0] == '-> top level has not enough items to select from'
+        error = exc.value.args[0]
+        assert  '-> top level has not enough items to select from' in error
 
     def test_deal_with_recursion(self):
-        data = {'a': {'a':'a', 'b':{'a': 'b', 'c':'c', 'd':1}}}
+        data = {'a': {'a': 'a', 'b': {'a': 'b', 'c': 'c', 'd': 1}}}
         schema = ('a', (('a', 'a'), ('b', recursive.AllObjects((types.string, types.string)))))
         with raises(Invalid) as exc:
             validator = engine.Validator(data, schema)
             validator.validate()
 
-        assert exc.value.args[0] == '-> a -> b -> d  did not pass validation against callable: string'
+        error = exc.value.args[0]
+        assert '1  did not pass validation against callable: string' in error
+        assert 'not of type string' in error
 
 
 class TestIterableValidator(object):
@@ -210,8 +207,8 @@ class TestIterableValidator(object):
         iter_validator = engine.IterableValidator(data, schema, index=100)
         with raises(SchemaError) as exc:
             iter_validator.validate()
-
-        assert exc.value.args[0] == '-> top level has not enough items to select from'
+        error = exc.value.args[0]
+        assert '-> top level has not enough items to select from' in error
 
     def test_validate_nested_array_first(self):
         data = [{'a':'a'}, {'b': 'b'}]
@@ -220,7 +217,8 @@ class TestIterableValidator(object):
         with raises(Invalid) as exc:
             iter_validator.validate()
 
-        assert exc.value.args[0] == "-> list[0] -> a -> a  did not match 'b'"
+        error = exc.value.args[0]
+        assert  "-> list[0] -> a -> a  did not match 'b'" in error
 
     def test_validate_nested_array(self):
         data = [{'a':'b'}, {'b': 'c'}]
@@ -229,7 +227,8 @@ class TestIterableValidator(object):
         with raises(Invalid) as exc:
             iter_validator.validate()
 
-        assert exc.value.args[0] == "-> list[1] -> b -> c  did not match 'b'"
+        error = exc.value.args[0]
+        assert  "-> list[1] -> b -> c  did not match 'b'" in error
 
     def test_report_schema_errors(self):
         data = [1,2,3,4]
@@ -237,14 +236,33 @@ class TestIterableValidator(object):
         iter_validator = engine.IterableValidator(data, schema, index=0)
         with raises(SchemaError) as exc:
             iter_validator.validate()
+        error = exc.value.args[0]
+        assert "iterable contains single items, schema does not" in error
 
-        assert exc.value.args[0] == "-> top level iterable contains single items, schema does not"
-
-    def test_schema_error_on_non_list(self):
+    def test_invalid_on_non_list(self):
         data = {'foo': [{'a':'b'}, {'b': 'c'}]}
         schema = (types.string, 'b') # note how this is not normalized
         iter_validator = engine.IterableValidator(data, schema, index=0)
-        with raises(SchemaError) as exc:
+        with raises(Invalid) as exc:
             iter_validator.validate()
+        error = exc.value.args[0]
+        assert "top level did not pass validation against callable: IterableValidator" in error
 
-        assert exc.value.args[0] == "-> top level IterableValidator needs a list to validate"
+
+class TestOptionalKeys(object):
+
+    def test_optional_key_passes(self):
+        data = {'a': 1, 'c': 3}
+        schema = (('a', 1), (optional('b'), 2), ('c', 3))
+        validator = engine.Validator(data, schema)
+        assert validator.validate() is None
+
+    def test_optional_key_raises(self):
+        data = {'a': 1, 'b': 3, 'c': 3}
+        schema = (('a', 1), (optional('b'), 2), ('c', 3))
+        validator = engine.Validator(data, schema)
+        with raises(Invalid) as exc:
+            validator.validate()
+        exc_msg = str(exc.value)
+        assert '3  did not match 2' in exc_msg
+        assert 'b -> 3' in exc_msg
