@@ -32,7 +32,6 @@ class TestAnyItem(object):
         msg = "-> list[]  did not contain any valid items matching 'a'"
         assert exc.value.args[0] == msg
 
-
     def test_any_item_pass(self):
         data = [1, 2, 6, 4, 5]
         schema = 6
@@ -45,6 +44,24 @@ class TestAnyItem(object):
         any_item = iterables.AnyItem(schema)
         assert  any_item(data, []) is None
 
+    def test_any_item_fail_on_non_lists(self):
+        data = 4
+        schema = 1
+        all_items = iterables.AnyItem(schema)
+        with raises(Invalid) as exc:
+            all_items(data, [])
+        error = exc.value.args[0]
+        assert  'top level did not pass validation against callable: AnyItem' in error
+
+    def test_expecting_list_error(self):
+        data = 4
+        schema = 1
+        all_items = iterables.AnyItem(schema)
+        with raises(Invalid) as exc:
+            all_items(data, [])
+        assert 'expected a list but got int' == exc.value.reason
+
+
 
 class TestAllItems(object):
 
@@ -54,13 +71,31 @@ class TestAllItems(object):
         all_items = iterables.AllItems(schema)
         assert all_items(data, []) is None
 
+    def test_all_items_fail_on_non_lists(self):
+        data = 4
+        schema = 1
+        all_items = iterables.AllItems(schema)
+        with raises(Invalid) as exc:
+            all_items(data, [])
+        error = exc.value.args[0]
+        assert  'top level did not pass validation against callable: AllItems' in error
+
+    def test_expecting_list_error(self):
+        data = 4
+        schema = 1
+        all_items = iterables.AllItems(schema)
+        with raises(Invalid) as exc:
+            all_items(data, [])
+        assert 'expected a list but got int' == exc.value.reason
+
     def test_all_items_fail(self):
         data = [1, 1, 3, 1]
         schema = 1
         all_items = iterables.AllItems(schema)
         with raises(Invalid) as exc:
             all_items(data, [])
-        assert exc.value.args[0] == '-> list[2] item did not match 1'
+        error = exc.value.args[0]
+        assert  '-> list[2] item did not match 1' in error
 
     def test_all_items_fail_last_item(self):
         data = [1, 1, 1, 3]
@@ -68,7 +103,9 @@ class TestAllItems(object):
         all_items = iterables.AllItems(schema)
         with raises(Invalid) as exc:
             all_items(data, [])
-        assert exc.value.args[0] == '-> list[3] item did not match 1'
+
+        error = exc.value.args[0]
+        assert  '-> list[3] item did not match 1' in error
 
     def test_all_items_fail_uses_first_invalid(self):
         data = [3, 5, 2, 3]
@@ -76,5 +113,6 @@ class TestAllItems(object):
         all_items = iterables.AllItems(schema)
         with raises(Invalid) as exc:
             all_items(data, [])
-        assert exc.value.args[0] == '-> list[0] item did not match 1'
 
+        error = exc.value.args[0]
+        assert  '-> list[0] item did not match 1' in error
